@@ -2,29 +2,80 @@ import re
 from datetime import datetime, timedelta
 
 from .const import DEFAULT_KW_ROUND_PERSICION, MeterType
+from .error import APIChangedError
 from .helpers import convert_to_kw
 
 
 class Response(object):
     """Basic Response object that can be constructed from a json response"""
 
-    def __init__(self, json_response: dict):
+    # A list of attributes that should be in the json_response
+    _JSON_ATTRS = []
+    # A list of attributes that may be in the json_response but aren"t required
+    _OPTIONAL_JSON_ATTRS = []
+
+    def __init__(self, json_response: dict, no_check: bool = False):
         self.json_response = json_response
-        self._set_attrs()
+        self._set_attrs(no_check)
+        self.response_validated = not no_check
 
-    def _set_attrs(self):
+    def _set_attrs(self, no_check: bool = False) -> None:
+        """Set attributes from _JSON_ATTRS as object properties. 
+        Also checks wether the response is valid. 
+        This can be disabled by passing no_check=True"""
+        missing_attrs = []
         for attr in self.__class__._JSON_ATTRS:
-            self._add_attr(attr)
+            self._add_attr(attr, missing_attrs)
 
-    def _add_attr(self, attr: str):
+        if len(self.__class__._OPTIONAL_JSON_ATTRS) > 0:
+            for attr in self.__class__._OPTIONAL_JSON_ATTRS:
+                self._add_attr(attr)
+
+        if not no_check:
+            added_attrs = self._get_added_attrs()
+
+        # We are missing some attributes in the json_response
+        if not no_check and len(missing_attrs) > 0:
+            raise APIChangedError(
+                self.__class__, self.json_response, added_attrs, missing_attrs
+            )
+
+    def _add_attr(self, attr, missing_attrs=[]) -> None:
+        # Make sure the attribute also exist in the json_response
         if attr in self.json_response:
             setattr(self, attr, self.json_response[attr])
         else:
-            raise ValueError(
-                f"Missing key '{attr}' in response from Powerwall. Either the Powerwall sent an invalid response or the API changed!"
-            )
+            missing_attrs.append(attr)
 
-    def __repr__(self):
+    def _get_added_attrs(self) -> list:
+        added_attrs = []
+        for attr in self.json_response.keys():
+            if attr not in self.__class__._JSON_ATTRS:
+                added_attrs.append(attr)
+        return added_attrs
+
+    # Helper methods to make interaction with optional json attributes easier
+
+    def has_optional_attrs(self) -> bool:
+        return len(self.__class__._OPTIONAL_JSON_ATTRS) > 0
+
+    def has_optional_attrs_set(self) -> bool:
+        """Checks wether all optional attributes are present in the json response"""
+        return set(
+            self.__class__._OPTIONAL_JSON_ATTRS + self.__class__._JSON_ATTRS
+        ) == set(self.json_response.keys())
+
+    def get(self, key, default=None):
+        """Equavivalent to dict.get"""
+        return self.json_response.get(key, default)
+
+    def has_key(self, key) -> bool:
+        return key in self.json_response.keys()
+
+    def __getitem__(self, key):
+        return getattr(self, key)
+
+    def __repr__(self) -> str:
         return str(self.json_response)
 
 
@@ -49,8 +100,8 @@ class MetersResponse(Response):
         "timeout",
     ]
 
-    def __init__(self, json_response: dict, meter: MeterType = None):
-        super().__init__(json_response)
+    def __init__(self, json_response: dict, meter: MeterType = None, no_check=False):
+        super().__init__(json_response, no_check=no_check)
         self.meter = meter
 
     def is_sending_to(self, precision=DEFAULT_KW_ROUND_PERSICION):
@@ -79,12 +130,14 @@ class MetersAggregateResponse(Response):
     Response for "meters/aggregates"
     """
 
-    def __init__(self, json_response):
+    def __init__(self, json_response, no_check=False):
         self.json_response = json_response
         for meter in MeterType:
             if meter.value in json_response:
                 setattr(
-                    self, meter.value, MetersResponse(json_response[meter.value], meter)
+                    self,
+                    meter.value,
+                    MetersResponse(json_response[meter.value], meter, no_check),
                 )
 
     def get(self, meter: MeterType) -> MetersResponse:
@@ -94,9 +147,11 @@ class MetersAggregateResponse(Response):
 class MeterDetailsResponse(Response):
     _JSON_ATTRS = ["id", "location", "type", "cts", "inverted", "connection"]
 
-    def __init__(self, json_response):
-        super().__init__(json_response)
-        self.cached_readings = MetersResponse(json_response["Cached_readings"])
+    def __init__(self, json_response, no_check=False):
+        super().__init__(json_response, no_check)
+        self.cached_readings = MetersResponse(
+            json_response["Cached_readings"], no_check=no_check
+        )
 
 
 class SitemasterResponse(Response):
@@ -119,12 +174,11 @@ class SiteInfoResponse(Response):
         "grid_phase_setting",
         "country",
         "state",
-        "distributor",
-        "utility",
-        "retailer",
         "region",
     ]
 
+    _OPTIONAL_JSON_ATTRS = ["distributor", "utility", "retailer"]
+
 
 class CustomerRegistrationResponse(Response):
     _JSON_ATTRS = [
@@ -145,7 +199,7 @@ class PowerwallStatusResponse(Response):
 
     _JSON_ATTRS = ["start_time", "up_time_seconds", "is_new", "version", "git_hash"]
 
-    def __init__(self, json_response):
+    def __init__(self, json_response, no_check=False):
         self.json_response = json_response
 
         resp = json_response
@@ -187,11 +241,28 @@ class PowerwallsStatusResponse(Response):
 
 
 class ListPowerwallsResponse(Response):
+    class PowerwallResponse(Response):
+        _JSON_ATTRS = [
+            "PackagePartNumber",
+            "PackageSerialNumber",
+            "type",
+            "grid_state",
+            "grid_reconnection_time_seconds",
+            "under_phase_detection",
+            "updating",
+            "commissioning_diagnostic",
+            "update_diagnostic",
+        ]
+
     _JSON_ATTRS = ["powerwalls", "has_sync", "sync", "states"]
 
-    def __init__(self, json_response):
-        super().__init__(json_response)
-        self.status = PowerwallsStatusResponse(self.json_response)
+    def __init__(self, json_response, no_check=False):
+        super().__init__(json_response, no_check)
+        self.status = PowerwallsStatusResponse(self.json_response, no_check=no_check)
+        self.powerwalls = [
+            ListPowerwallsResponse.PowerwallResponse(powerwall, no_check=no_check)
+            for powerwall in self.powerwalls
+        ]
 
 
 class SolarsResponse(Response):
